import { a as LogLevel, g as LogHandlerOptions, d as LogHandler, c as LogMessage } from './log-b80164f3.js';

declare type BinInput = Uint8Array | ArrayBuffer | string | number[];
declare function _encodeUtf8Polyfill(str: string): Uint8Array;
declare function _decodeUtf8Polyfill(buf: Uint8Array): string;
declare function stringToUInt8Array(text: string): Uint8Array;
declare function Uint8ArrayToString(bin: Uint8Array): string;
declare function toUint8Array(data: BinInput): Uint8Array;
declare function toHex(bin: BinInput): string;
declare function toBase64(bin: BinInput): string;
declare function toBase64Url(bin: BinInput): string;
declare function fromBase64(s: string): Uint8Array;
/** Compare contents of binary arrays */
declare function equalBinary(a: ArrayBuffer | Uint8Array, b: ArrayBuffer | Uint8Array): boolean;
declare function jsonToUint8Array(json: any): Uint8Array | never;
declare function Uint8ArrayToJson<T = any>(data: Uint8Array): T | never;

declare function randomUint8Array(length?: number): Uint8Array;
declare function digest(message: BinInput, algorithm?: AlgorithmIdentifier): Promise<Uint8Array>;
declare function deriveKeyPbkdf2(secret: BinInput, opt?: {
    iterations?: number;
    salt?: BinInput;
}): Promise<CryptoKey>;
declare function encrypt(data: Uint8Array, key: CryptoKey): Promise<Uint8Array>;
declare function decrypt(data: Uint8Array, key: CryptoKey): Promise<Uint8Array>;

declare function csv(data: any[], headerRow: string[]): string;

declare type Json = string | number | boolean | null | {
    [property: string]: Json;
} | Json[];
interface ObjectStorage<T = any> {
    setItem(key: string, value: T): void;
    getItem(key: string): T | undefined;
    removeItem(key: string): void;
    clear(): void;
    allKeys(): string[];
}
declare type NestedArray<T> = T | Array<NestedArray<T>>;

declare function arrayUnique<T>(x: T[]): T[];
declare function arrayMinus<T>(x: T[], y: T[]): T[];
declare function arrayUnion<T>(...a: T[][]): T[];
/** `[1,[2,3]]` becomes `[1,2,3]` */
declare function arrayFlatten<T>(...list: NestedArray<T>[]): T[];
declare function arrayIntersection<T>(x: T[], y: T[]): T[];
declare function arraySymmetricDifference<T>(x: T[], y: T[]): T[];
declare function arrayRemoveElement<T>(arr: T[], el: T): T[];
/** Only have it once in the set */
declare function arraySetElement<T>(arr: T[], el: T): T[];
declare function arrayFilterInPlace<T>(array: T[], fn: (el: T) => boolean): T[];
declare function arrayToggleInPlace<T>(array: T[], el: T): T[];
declare function arrayEmptyInPlace<T>(array: T[]): T[];
declare function arraySorted<T>(arr: Iterable<T> | ArrayLike<T>, cond?: ((a: T, b: T) => number) | undefined): T[];
declare function arraySortedNumbers(arr: number[]): number[];
declare function arrayIsEqual<T>(array1: T[], array2: T[]): boolean;
declare function arrayShuffleInPlace<T>(array: T[]): T[];
declare function arrayShuffle<T>(array: T[]): T[];
/** Randomly shuffle the order of the array's elements. Force to have a different order if array has more than one element. */
declare function arrayShuffleForce<T>(array: T[]): T[];
declare function arrayRandomElement<T>(array: T[]): T;
declare function arrayMax<T>(...array: NestedArray<T>[]): T;
declare function arrayMin<T>(...array: NestedArray<T>[]): T;
declare function arraySum(...array: NestedArray<number>[]): number;
declare function arrayAvg(...array: NestedArray<number>[]): number;
declare function arrayBatches<T>(array: T[], chunckLength: number): T[][];
declare function createArray<T>(size?: number, item?: T | ((index: number) => T)): T[];

declare function useBase(alphaOrBase: string | number): {
    encode: (source: BinInput, padToLength?: number) => string;
    decode: (source: string, padToLength?: number) => Uint8Array;
};
declare const encodeBase16: (source: BinInput, padToLength?: number) => string;
declare const decodeBase16: (source: string, padToLength?: number) => Uint8Array;
declare const encodeBase32: (source: BinInput, padToLength?: number) => string;
declare const decodeBase32: (source: string, padToLength?: number) => Uint8Array;
declare const encodeBase58: (source: BinInput, padToLength?: number) => string;
declare const decodeBase58: (source: string, padToLength?: number) => Uint8Array;
declare const encodeBase62: (source: BinInput, padToLength?: number) => string;
declare const decodeBase62: (source: string, padToLength?: number) => Uint8Array;
declare const encodeBase64: (source: BinInput, padToLength?: number) => string;
declare const decodeBase64: (source: string, padToLength?: number) => Uint8Array;

declare const toCamelCase: (s: string) => string;
declare function toCapitalize(s: string): string;
declare function toCapitalizeWords(s: string): string;
declare function fromCamelCase(str: string, separator?: string): string;

declare function stringToBoolean(value?: string, defaultValue?: boolean): boolean;
declare function stringToInteger(value?: string, defaultValue?: number): number;
declare function stringToFloat(value?: string, defaultValue?: number): number;
declare function valueToBoolean(value?: any, defaultValue?: boolean): boolean;
declare function valueToInteger(value?: any, defaultValue?: number): number;
declare function valueToFloat(value?: any, defaultValue?: number): number;
declare function valueToString(value?: any, defaultValue?: string): string;
declare const toFloat: typeof valueToFloat;
declare const toInt: typeof valueToInteger;
declare const toString: typeof valueToString;
declare const toBool: typeof valueToBoolean;
declare type RenderMessagesOptions = {
    trace?: boolean;
    pretty?: boolean;
};
declare function formatMessages(messages: any[], opt?: RenderMessagesOptions): any[];
declare function renderMessages(messages: any[], opt?: RenderMessagesOptions): string;
declare function fixBrokenUth8String(brokenString: string): string;

declare type CurrencyInput = number | string | Currency;
declare type CurrencyFormat = (currency?: Currency, opts?: CurrencyOptions) => string;
interface CurrencyOptions {
    symbol?: string;
    separator?: string;
    decimal?: string;
    errorOnInvalid?: boolean;
    precision?: number;
    increment?: number;
    useVedic?: boolean;
    pattern?: string;
    negativePattern?: string;
    format?: CurrencyFormat;
    fromCents?: boolean;
    groups?: RegExp;
}
/** @deprecated use decimal() instead */
declare function currency(value: CurrencyInput, opts?: CurrencyOptions): Currency;
/**
 * Immuteable currency representation
 * @deprecated use decimal() instead
 */
declare class Currency {
    readonly intValue: number;
    readonly value: number;
    private readonly _settings;
    private readonly _precision;
    constructor(value: CurrencyInput, opts?: CurrencyOptions);
    add(number: CurrencyInput): Currency;
    subtract(number: CurrencyInput): Currency;
    multiply(number: CurrencyInput): Currency;
    divide(number: CurrencyInput): Currency;
    distribute(count: number): number[];
    dollars(): number;
    cents(): number;
    format(options: CurrencyOptions | Function): any;
    toString(): string;
    toJSON(): number;
    static zero: Currency;
    static one: Currency;
    static hundred: Currency;
    static sum(...array: (CurrencyInput | CurrencyInput[])[]): Currency;
    static avg(...array: (CurrencyInput | CurrencyInput[])[]): Currency;
}

declare const DAY_MS = 86400000;
declare type DayValue = number;
declare type DayInput = DayValue | number | string | Date | [number, number?, number?];
declare function dayYear(day: DayValue): DayValue;
declare function dayMonth(day: DayValue): DayValue;
declare function dayDay(day: DayValue): DayValue;
declare function dayToParts(day: DayValue): [number, number, number];
declare function dayToDate(day: DayValue, gmt?: boolean): Date;
declare function dayFromToday(): DayValue;
declare function dayFromAny(value: DayInput, gmt?: boolean): DayValue | undefined;
declare function dayToDateGMT(day: DayValue): Date;
declare function dayFromDate(date: Date, gmt?: boolean): DayValue;
declare function dayFromDateGMT(date: Date): DayValue;
declare function dayToTimestamp(day: DayValue, gmt?: boolean): number;
declare function dayFromTimestamp(ms: number, gmt?: boolean): DayValue;
declare function dayToString(day: DayValue, sep?: string): string;
declare function dayFromParts(year: number, month?: number, day?: number): DayValue | undefined;
declare function dayFromString(value: string): DayValue | undefined;
declare function dayMonthStart(day: DayValue, offset?: number): DayValue;
declare function dayYearStart(day: DayValue, offset?: number): DayValue;
declare function dayOffset(day: DayValue, offset: number): DayValue;
declare function dayDiff(left: DayValue, right: DayValue): number;

declare type DayInputLegacy = DayInput | Day;
/**
 * Date represented as positive integer value. Years smaller 0 are not supported.
 * @deprecated
 */
declare class Day {
    days: DayValue;
    get value(): number;
    constructor(days?: DayInputLegacy);
    static fromNumber(n: number): Day;
    static fromString(dateString: string): Day | undefined;
    static fromDate(date: Date, gmt?: boolean): Day;
    static fromDateGMT(date: Date): Day;
    static from(value: DayInputLegacy, gmt?: boolean): Day | undefined;
    toNumber(): number;
    /** Just for future extensions */
    toJson(): number;
    toString(sep?: string): string;
    toDate(gmt?: boolean): Date;
    toDateGMT(): Date;
    get year(): number;
    get month(): number;
    get day(): number;
    dayOffset(offset: number): Day;
    monthStart(): Day;
    yearStart(): Day;
    /** Very stupid approach, only works for days <= 28 */
    monthOffset(offset: number): Day;
    daysUntil(otherDay: DayInputLegacy): number;
    yesterday(): Day;
    tomorrow(): Day;
}
declare function forEachDay(from: DayInputLegacy, to: DayInputLegacy, handler: (date: Day) => Promise<void> | void): Promise<void>;
declare function today(): Day;
/** @deprecated */
declare function day(days?: DayInputLegacy): Day;
declare function dateStringToDays(dateString: string): number;

declare type DecimalValue = number;
declare type DecimalInput = number | string | DecimalValue;
declare function decimal(value: DecimalInput, decimalPlaces?: number): DecimalValue;
declare function decimalFromCents(value: DecimalInput, decimalPlaces?: number): DecimalValue;
declare function decimalToCents(value: DecimalInput, decimalPlaces?: number): number;
declare function decimalCentsPart(value: DecimalInput, decimalPlaces?: number): DecimalValue;

declare function deepEqual(a: any, b: any, hash?: WeakSet<object>): boolean;
declare function deepMerge(target: any, ...sources: any[]): any;

declare const escapeHTML: (s: string) => string;
declare const unescapeHTML: (s: string) => string;

declare type Primitive = null | undefined | string | number | boolean | symbol | bigint;
declare function isObject(obj: unknown): obj is object;
declare function isPrimitive(obj: unknown): obj is Primitive;
declare function isArray(obj: unknown): obj is Array<any>;
declare function isRecord(obj: unknown): obj is Record<string, any>;
declare function isString(obj: unknown): obj is string;
declare function isNumber(obj: unknown): obj is number;
declare function isInteger(obj: unknown): obj is number;
declare function isSafeInteger(obj: unknown): obj is number;
declare function isBoolean(obj: unknown): obj is boolean;
declare function isNullOrUndefined(obj: unknown): obj is null | undefined;

declare type EntryProcessor = (key: string, value: any) => any;
/**
 * Similar to JSON.stringify but can handle circular references
 */
declare function jsonStringify(obj: any, replacer?: EntryProcessor | null, spaces?: string | number | null, cycleReplacer?: EntryProcessor): string;

declare function getSecureRandom(): number;
declare function getSecureRandomIfPossible(): number;
declare function randomBoolean(bias?: number): boolean;
/** max is not included, min is included */
declare function randomInt(max?: number, min?: number): number;
declare function randomFloat(max?: number, min?: number): number;
declare function between(min: number, value: number, max: number): number;
/** See also arraySum */
declare function sum(array: number[]): number;
/** See also arrayAvg */
declare function avg(array: number[]): number;
/** Fancy prime number check ;) */
declare function isPrimeRX(value: number): boolean;
declare function isPrime(value: number): boolean;

declare function parseOrderby(value?: string): {
    field: string;
    orderby: string;
    asc: boolean;
    desc: boolean;
};
declare function composeOrderby(field: string, asc?: boolean): string;
declare function cmp(a: any, b: any, asc?: boolean): number;
declare function sortedOrderby<T>(values: T[], ...orderby: string[]): T[];

declare function toValidFilename(string: string): string | never;

declare function escapeRegExp(value: RegExp | string): string;

declare type RoundingMode = (value: number) => number;
declare const isHalf: (value: number) => boolean;
declare const isEven: (value: number) => boolean;
declare const roundUp: RoundingMode;
declare const roundDown: RoundingMode;
declare const roundHalfUp: RoundingMode;
/**
 * Round a number with half values to nearest odd integer.
 */
declare const roundHalfOdd: RoundingMode;
/**
 * Round a number with half values to nearest integer farthest from zero.
 */
declare const roundHalfAwayFromZero: RoundingMode;
/**
 * Round a number with half values down.
 */
declare const roundHalfDown: RoundingMode;
/**
 * Round a number with half values to nearest even integer.
 * https://wiki.c2.com/?BankersRounding
 */
declare const roundHalfEven: RoundingMode;
/**
 * Round a number with half values to nearest integer closest to zero.
 */
declare const roundHalfTowardsZero: RoundingMode;

interface SortableItem {
    sort_weight: number;
}
declare function startSortWeight(items: SortableItem[]): number;
declare function endSortWeight(items: SortableItem[]): number;
declare function moveSortWeight(newIndex: number, oldIndex: number, items: SortableItem[]): number;
declare function sortedItems<T extends SortableItem>(items: T[]): T[];

declare function linkifyPlainText(text: string): string;
declare function toHumanReadableUrl(url: string): string;
declare function encodeQuery(data: Record<string, any>): string;
declare function parseQuery(queryString: string): any;

/**
 * Call a create function if key does not yet exist on an object. Returns the found or created object. Example:
 *
 * ```js
 * function createRoom(room, rooms) { return new Room() }
 * ensureKey(rooms, room, createRoom).enter()
 * ```
 */
declare function ensureKey<T>(obj: Record<string, T>, key: string, createFn: (key?: string, obj?: Record<string, T>) => T): T;
/**
 * Call a create function if key does not yet exist on an object. Returns the found or created object. Example:
 *
 * ```js
 * async function fetchItem(id, cache) { ... }
 * let data = await ensureKey(cache, id, fetchItem)
 * ```
 */
declare function ensureKeyAsync<T>(obj: Record<string, T>, key: string, createFn: (key?: string, obj?: Record<string, T>) => Promise<T>): Promise<T>;
declare function size(obj: any): any;
declare function last<T>(array?: T[]): T | undefined;
declare function empty(value: any): boolean;
declare function cloneObject<T>(obj: T): T;
declare function cloneJsonObject<T = Json>(obj: T): T;
/**
 * Cache result of a function. Same arguments have to always return the same result in order to get expected optimization!
 *
 * ```
 * const square = memoize((value) => value * value)`
 * square(2) // == 2
 * ```
 */
declare function memoize<In extends any, Out extends any>(fn: (arg: In) => Out): (arg: In) => Out;
/** Repeat `count` times. Starts with `0` */
declare function forTimes<T = undefined>(count: number, fn: (i: number, count: number) => T): T[];

declare function regExpString(rx: string | RegExp): string;
declare function regExpEscape(str: string): string;
declare class XRX {
    lookBehind: {
        rx: RegExp;
        expect: boolean;
    } | undefined;
    namedGroups: Record<string, number>;
    names: string[];
    rx: RegExp;
    constructor(pattern: string | RegExp, flags?: string);
    get lastIndex(): number;
    set lastIndex(index: number);
    _handleMatch(m: any): any;
    exec(str: string): any;
    execAll(str: string): any[];
    replace(str: string, replacement: string | Function): string;
}

declare type DisposerFunction = () => any | Promise<any>;
declare type Disposer = DisposerFunction | {
    dispose?: Function | Promise<unknown>;
    cleanup?: Function | Promise<unknown>;
};
interface Disposable {
    dispose(): void | Promise<void>;
}
interface UseDisposeConfig {
    name?: string;
}
declare function useDispose(config?: string | UseDisposeConfig): (() => Promise<void>) & {
    track: (obj?: Disposer | undefined) => DisposerFunction | undefined;
    add: (obj?: Disposer | undefined) => DisposerFunction | undefined;
    untrack: (disposable: Disposer) => Promise<void>;
    dispose: () => Promise<void>;
    exec: () => Promise<void>;
    getSize(): number;
};
declare type UseDispose = ReturnType<typeof useDispose>;
/** @deprecated use `useDispose` instead */
declare const useDisposer: typeof useDispose;
declare function useDefer(config?: {
    mode?: "lifo" | "fifo";
}): ((expectSync?: boolean) => Promise<void>) & {
    add: (obj: Disposer) => void;
    exec: (expectSync?: boolean) => Promise<void>;
    getSize(): number;
};
declare type UseDefer = ReturnType<typeof useDefer>;
declare function useTimeout(fn: Function, timeout?: number): DisposerFunction;
declare function useInterval(fn: Function, interval: number): DisposerFunction;
declare function useEventListener(emitter: any, eventName: string, fn: (ev?: any) => void, ...args: any[]): DisposerFunction;

declare type Mutex = (fn: Function, elseFn?: Function) => boolean;
declare type AsyncMutex = (fn: Function, elseFn?: Function) => Promise<boolean>;
declare function useMutex(): Mutex;
declare function useAsyncMutex(): AsyncMutex;

declare type EmitterHandler = (...objs: any[]) => void;
declare type EmitterAllHandler<T = string> = (key: T, ...objs: any[]) => void;
declare type ListenerSignature<L> = {
    [E in keyof L]: (...args: any[]) => any;
};
declare type DefaultListener = {
    [k: string]: (...args: any[]) => any;
};
declare class Emitter<RemoteListener extends ListenerSignature<RemoteListener> = DefaultListener, LocalListener extends ListenerSignature<LocalListener> = RemoteListener> implements Disposable {
    subscribers: any;
    subscribersOnAny: any[];
    call: RemoteListener;
    emit<U extends keyof RemoteListener>(event: U, ...args: Parameters<RemoteListener[U]>): Promise<boolean>;
    onAny(fn: EmitterHandler): void;
    on<U extends keyof LocalListener>(event: U, listener: LocalListener[U]): DisposerFunction;
    onCall(handlers: Partial<LocalListener>): void;
    once<U extends keyof LocalListener>(event: U, listener: LocalListener[U]): DisposerFunction;
    off<U extends keyof LocalListener>(event: U, listener: LocalListener[U]): this;
    removeAllListeners(): this;
    dispose(): void;
}
declare global {
    interface ZeedGlobalContext {
        emitter?: Emitter;
    }
    interface ZeedGlobalEmitter {
    }
}
/** Global emitter that will listen even across modules */
declare function getGlobalEmitter(): Emitter<ZeedGlobalEmitter>;
declare const messages: Emitter<DefaultListener, DefaultListener>;
declare function lazyListener(emitter: any, listenerKey?: string): (key?: string, skipUnmatched?: boolean) => Promise<any>;

interface PoolConfig {
    maxParallel?: number;
}
declare type PoolTaskFn<T = any> = (taskInfo?: PoolTask<T>) => Promise<T>;
declare enum PoolTaskState {
    waiting = 0,
    running = 1,
    finished = 2
}
interface PoolTask<T> {
    readonly id: string;
    readonly task: PoolTaskFn<T>;
    readonly done: Function;
    readonly setMax: (max: number) => void;
    readonly setResolved: (resolved: number) => void;
    readonly incResolved: (inc?: number) => void;
    state: PoolTaskState;
    priority: number;
    max: number;
    resolved: number;
    result?: T;
    payload?: unknown;
}
declare enum PoolTaskIdConflictResolution {
    replace = 0,
    memoize = 1,
    error = 2
}
interface PoolTaskEvents {
    didUpdate(max: number, resolved: number, presentMax: number, presentResolved: number): void;
    didStart(id: string): void;
    didCancel(id: string): void;
    didFinish(): void;
    didResolve(id: string, value: any): void;
    didReject(id: string, error: any): void;
}
declare function usePool<T = any>(config?: PoolConfig): {
    events: Emitter<PoolTaskEvents, PoolTaskEvents>;
    cancel: (id: string) => void;
    cancelAll: () => void;
    enqueue: <P>(task: PoolTaskFn<T>, config?: {
        id?: string | undefined;
        max?: number | undefined;
        resolved?: number | undefined;
        idConflictResolution?: PoolTaskIdConflictResolution | undefined;
        payload?: P | undefined;
    }) => {
        id: string;
        promise: Promise<any>;
        dispose: () => void;
        cancel: () => void;
    };
    dispose: () => void;
    waitFinishAll: () => Promise<unknown>;
};
declare type Pool = ReturnType<typeof usePool>;

declare function createPromise<T>(): [Promise<T>, any, any];
/** Sleep for `milliSeconds`. Example 1s: `await sleep(1000)` */
declare function sleep(milliSeconds: number): Promise<void>;
declare function immediate(): Promise<void>;
declare const timeoutReached: unique symbol;
declare function timeout<T>(promise: Promise<T>, milliSeconds: number, timeoutValue?: symbol): Promise<T | typeof timeoutValue>;
declare const timoutError: Error;
declare function isTimeout(value: any): boolean;
declare function tryTimeout<T>(promise: Promise<T>, milliSeconds: number): Promise<T | undefined>;
/** Wait for `event` on `obj` to emit. Resolve with result or reject on `timeout` */
declare function waitOn(obj: any, event: string, timeoutMS?: number): Promise<any>;
declare function isPromise<T>(value: Promise<T> | T): value is Promise<T>;
/** This is exactly what Prose.resolve(x) is supposed to be: return a Promise no matter what type x is */
declare function promisify<T>(value: Promise<T> | T): Promise<T>;

declare type TaskFn<T = any> = () => Promise<T>;
interface TaskEvents {
    didUpdate(max: number, resolved: number): void;
    didStart(max: number): void;
    didCancel(): void;
    didFinish(): void;
}
/** Guarentee serial execution of tasks. Able to wait, pause, resume and cancel all. */
declare class SerialQueue extends Emitter<TaskEvents> {
    private queue;
    private waitToFinish;
    private currentTask?;
    private log;
    private countMax;
    private countResolved;
    private paused;
    name: string;
    constructor(opt?: {
        name?: string;
        logLevel?: LogLevel;
    });
    private performNext;
    /** Enqueue task to be executed when all other tasks are done. Except `immediate = true`. */
    enqueue<T>(task: TaskFn<T>, opt?: {
        immediate?: boolean;
        name?: string;
    }): Promise<T>;
    /** If a task is already performing, execute immediately. Otherwise enqueue as usual. */
    enqueueReentrant<T>(task: TaskFn<T>, opt?: {
        name?: string;
    }): Promise<T>;
    /** Remove all tasks from queue that are not yet executing. */
    cancelAll(unblock?: boolean): Promise<void>;
    /** Pause execution after current task is finished. */
    pause(): Promise<void>;
    /** Resume paused queue. */
    resume(): void;
    /** Wait for all tasks to finish */
    wait(): Promise<unknown>;
    get isPaused(): boolean;
    get hasTasks(): boolean;
}

interface ThrottleOptions {
    delay?: number;
    trailing?: boolean;
    leading?: boolean;
}
interface DebounceOptions {
    delay?: number;
}
/**
 * A special throttle implementation that tries to distribute execution
 * in an optimal way.
 *
 * For UI usage the function is executed on first occasion (`leading`).
 * If more calls follow it will again be executed at end (`trailing`).
 * If the next call is inside the timeframe, it is delayed until `trailing`.
 * This avoids timewise too close calls.
 * It is possible to `cancel` the timeout and to `flush` a call, e.g. if
 * leaving UI situation where a final call is required to write data or similar.
 */
declare function throttle<F extends Function>(callback: F, opt?: ThrottleOptions): F & {
    cancel: () => void;
    dispose: () => void;
};
/**
 * Debounce fits best for filtering a large peak of events.
 * For UI event filtering throttle is probably a better choice.
 */
declare function debounce<F extends Function>(callback: F, opt?: DebounceOptions): F & {
    cancel: () => void;
    dispose: () => void;
};

declare global {
    interface ZeedGlobalContext {
    }
}
declare function getGlobalContext(): ZeedGlobalContext;

declare function isLocalHost(hostname?: string): boolean;

/**
 * Very basic logger. Please take a look at the browser and node
 * optimized loggers. This one is just the absolute fallback option.
 *
 * @param level Log level
 * @returns Logger
 */
declare function LoggerConsoleHandler(opt?: LogHandlerOptions): LogHandler;

interface NamespaceFilter {
    (name: string): boolean;
    accept: RegExp[];
    reject: RegExp[];
    filter: string;
}
declare function getNamespaceFilterString(defaultNamespaceFilter: any): string;
/**
 * Filter as described here https://github.com/visionmedia/debug#wildcards
 *
 * @param filter Namespace filter
 * @returns Function to check if filter applies
 */
declare function useNamespaceFilter(filter?: string): NamespaceFilter;
declare function useLevelFilter(filter?: string | number): (level: LogLevel) => boolean;

/**
 * Collect messages in a list.
 *
 * @param level Log level
 * @returns Logger
 */
declare function LoggerMemoryHandler(opt: LogHandlerOptions & {
    messages: LogMessage[];
}): LogHandler;

/** See http://developer.mozilla.org/en-US/docs/Web/API/MessageEvent */
interface ChannelMessageEvent {
    data: any;
    origin?: string;
    lastEventId?: string;
}
/**
 * Inspired by
 * http://developer.mozilla.org/en-US/docs/Web/API/BroadcastChannel
 * https://deno.com/deploy/docs/runtime-broadcast-channel
 * */
declare abstract class Channel extends Emitter<{
    message(event: ChannelMessageEvent): void;
    messageerror(event: ChannelMessageEvent): void;
    connect(): void;
    disconnect(): void;
    close(): void;
}> {
    id: string;
    abstract isConnected?: boolean;
    abstract postMessage(data: any): void;
    close(): void;
}
/** Very basic channel demonstrating local communication */
declare class LocalChannel extends Channel {
    isConnected: boolean;
    other?: LocalChannel;
    postMessage(data: any): void;
}
declare function fakeWorkerPair(): [LocalChannel, LocalChannel];

interface Encoder {
    encode(data: any): Promise<Uint8Array>;
    decode(data: Uint8Array): Promise<any>;
}
declare class NoopEncoder implements Encoder {
    encode(data: any): Promise<Uint8Array>;
    decode(data: Uint8Array): Promise<any>;
}
declare class JsonEncoder implements Encoder {
    encode(data: any): Promise<Uint8Array>;
    decode(data: Uint8Array): Promise<any>;
}
declare class CryptoEncoder implements Encoder {
    key: CryptoKey;
    constructor(key: CryptoKey);
    encode(data: any): Promise<Uint8Array>;
    decode(data: Uint8Array): Promise<any>;
}

declare type MessageAction = {
    name: string;
    id: string;
    args?: Json[];
};
declare type MessageResult = {
    id: string;
    result?: Json;
    error?: {
        stack?: string;
        name: string;
        message: string;
    };
};
declare type Message = MessageAction | MessageResult;
declare type MessagesOptions = {
    timeout?: number;
};
declare type MessagesDefaultMethods<L> = {
    dispose(): void;
    connect?(channel: Channel): void;
    options(opt: MessagesOptions): L;
};
declare type MessagesMethods<L> = L & MessagesDefaultMethods<L>;
declare type MessageDefinitions = Record<any, (...args: any) => Promise<any>>;
declare type MessageHub = {
    dispose(): void;
    connect: (newChannel: Channel) => void;
    listen<L extends MessageDefinitions>(newHandlers: L): void;
    send<L extends MessageDefinitions>(): MessagesMethods<L>;
};
declare const createPromiseProxy: <P extends object>(fn: (name: string, args: any[], opt: any) => Promise<unknown>, opt: MessagesOptions, predefinedMethods?: any) => P;
declare function useMessageHub(opt?: {
    name?: string;
    channel?: Channel;
    encoder?: Encoder;
    retryAfter?: number;
    ignoreUnhandled?: boolean;
    debug?: boolean;
}): MessageHub;

interface PubSubConfig {
    channel: Channel;
    encoder?: Encoder;
    name?: string;
    debug?: boolean;
}
declare class PubSub<L extends ListenerSignature<L> = DefaultListener> extends Emitter<L> {
    name: string;
    channel: Channel;
    encoder: Encoder;
    log: any;
    debug: boolean;
    get shortId(): string;
    constructor(opt: PubSubConfig);
    private emitSuper;
    emit<U extends keyof L>(event: U, ...args: Parameters<L[U]>): Promise<boolean>;
    publish: <U extends keyof L>(event: U, ...args: Parameters<L[U]>) => Promise<boolean>;
    subscribe: <U extends keyof L>(event: U, listener: L[U]) => DisposerFunction;
}
declare function usePubSub<L extends ListenerSignature<L> = DefaultListener>(opt: PubSubConfig): PubSub<L>;

declare type ArgumentsType<T> = T extends (...args: infer A) => any ? A : never;
declare type ReturnType$1<T> = T extends (...args: any) => infer R ? R : never;
interface RPCOptions<Remote> {
    /** No return values expected */
    onlyEvents?: boolean;
    /** Names of remote functions that do not need response. */
    eventNames?: (keyof Remote)[];
    /** Function to post raw message */
    post: (data: any) => void;
    /** Listener to receive raw message */
    on: (fn: (data: any) => void) => void;
    /** Custom function to serialize data */
    serialize?: (data: any) => any;
    /** Custom function to deserialize data */
    deserialize?: (data: any) => any;
}
declare type RPCFn<T> = {
    /** Call the remote function and wait for the result. */
    (...args: ArgumentsType<T>): Promise<Awaited<ReturnType$1<T>>>;
    /** Send event without asking for response */
    asEvent(...args: ArgumentsType<T>): void;
};
declare type RPCReturn<RemoteFunctions> = {
    [K in keyof RemoteFunctions]: RPCFn<RemoteFunctions[K]>;
};
declare function useRPC<RemoteFunctions = {}, LocalFunctions = {}>(functions: LocalFunctions, options: RPCOptions<RemoteFunctions>): RPCReturn<RemoteFunctions>;

declare type httpMethod = "GET" | "POST" | "PUT" | "DELETE" | "HEAD" | "CONNECT" | "OPTIONS" | "TRACE" | "PATCH";
declare function fetchBasic(url: string | URL, fetchOptions?: any, fetchFn?: (input: RequestInfo, init?: RequestInit) => Promise<Response>): Promise<Response | undefined>;
declare function fetchJson<T = Json>(url: string | URL, opts?: any, fetchFn?: (input: RequestInfo, init?: RequestInit) => Promise<Response>): Promise<T | undefined>;
declare function fetchOptionsFormURLEncoded(data: Object, method?: httpMethod): {
    headers: {
        "Content-Type": string;
    };
    body: string;
    cache: string;
    redirect: string;
    method: httpMethod;
};
declare function fetchOptionsJson(data: Object, method?: httpMethod): {
    headers: {
        "Content-Type": string;
        Accept: string;
    };
    body: string;
    cache: string;
    redirect: string;
    method: httpMethod;
};
declare function fetchText(url: string | URL, opts?: any, fetchFn?: (input: RequestInfo, init?: RequestInit) => Promise<Response>): Promise<string | undefined>;

declare function getWindow(): any | undefined;
declare function getNavigator(): any | undefined;
declare function getGlobal(): any;
declare function detect(info?: {
    ios: boolean;
    macos: boolean;
    windows: boolean;
    beaker: boolean;
    electron: boolean;
    wkwebview: boolean;
    pwa: boolean;
    pwaInstalled: boolean;
    browser: boolean;
    node: boolean;
    worker: boolean;
    test: boolean;
    jest: boolean;
    macosNative: boolean;
    iosNative: boolean;
    appleNative: boolean;
    touch: boolean;
}): {
    ios: boolean;
    macos: boolean;
    windows: boolean;
    beaker: boolean;
    electron: boolean;
    wkwebview: boolean;
    pwa: boolean;
    pwaInstalled: boolean;
    browser: boolean;
    node: boolean;
    worker: boolean;
    test: boolean;
    jest: boolean;
    macosNative: boolean;
    iosNative: boolean;
    appleNative: boolean;
    touch: boolean;
};
declare const isBrowser: () => boolean;
declare const platform: {
    ios: boolean;
    macos: boolean;
    windows: boolean;
    beaker: boolean;
    electron: boolean;
    wkwebview: boolean;
    pwa: boolean;
    pwaInstalled: boolean;
    browser: boolean;
    node: boolean;
    worker: boolean;
    test: boolean;
    jest: boolean;
    macosNative: boolean;
    iosNative: boolean;
    appleNative: boolean;
    touch: boolean;
};
/**
 * Before closing the tab/window or quitting the node process
 * allow to do something important first
 */
declare function useExitHandler(handler: () => void): void;

interface MemStorageOptions {
    objectFromString?: (data: string) => any;
    objectToString?: (data: any) => string;
}
declare class MemStorage<T = Json> implements ObjectStorage<T> {
    private store;
    constructor(opt?: MemStorageOptions);
    setItem(key: string, value: T): void;
    getItem(key: string): T | undefined;
    removeItem(key: string): void;
    clear(): void;
    allKeys(): string[];
}

/**
 * @returns Timestamp in miliseconds
 */
declare const getTimestamp: () => number;
declare function formatMilliseconds(ms: number): string;
declare function parseDate(...dateCandidates: (string | Date)[]): Date | undefined;

declare function uuidBytes(): Uint8Array;
declare const uuid32bit: () => number;
declare function uuid(): string;
declare function uuidEncode(bytes: Uint8Array): string;
declare function uuidDecode(uuid: string): Uint8Array;
declare function uuidB32(): string;
declare function uuid32Encode(bytes: Uint8Array): string;
declare function uuid32Decode(uuid: string): Uint8Array;
declare function uname(name?: string): string;
declare function qid(): string;
declare const uuidv4: () => string;
declare function suidBytes(): Uint8Array;
declare function suid(): string;
declare function suidDate(id: string): Date;
declare function suidBytesDate(id: Uint8Array): Date;

export { Uint8ArrayToJson as $, arraySum as A, arrayAvg as B, arrayBatches as C, createArray as D, useBase as E, encodeBase16 as F, decodeBase16 as G, encodeBase32 as H, decodeBase32 as I, encodeBase58 as J, decodeBase58 as K, encodeBase62 as L, decodeBase62 as M, encodeBase64 as N, decodeBase64 as O, BinInput as P, _decodeUtf8Polyfill as Q, stringToUInt8Array as R, toUint8Array as S, toHex as T, Uint8ArrayToString as U, toBase64 as V, toBase64Url as W, fromBase64 as X, equalBinary as Y, jsonToUint8Array as Z, _encodeUtf8Polyfill as _, deriveKeyPbkdf2 as a, isArray as a$, toCamelCase as a0, toCapitalize as a1, toCapitalizeWords as a2, fromCamelCase as a3, stringToBoolean as a4, stringToInteger as a5, stringToFloat as a6, valueToBoolean as a7, valueToInteger as a8, valueToFloat as a9, dayFromTimestamp as aA, dayToString as aB, dayFromParts as aC, dayFromString as aD, dayMonthStart as aE, dayYearStart as aF, dayOffset as aG, dayDiff as aH, DayInputLegacy as aI, Day as aJ, forEachDay as aK, today as aL, day as aM, dateStringToDays as aN, DecimalValue as aO, DecimalInput as aP, decimal as aQ, decimalFromCents as aR, decimalToCents as aS, decimalCentsPart as aT, deepEqual as aU, deepMerge as aV, escapeHTML as aW, unescapeHTML as aX, Primitive as aY, isObject as aZ, isPrimitive as a_, valueToString as aa, toFloat as ab, toInt as ac, toString as ad, toBool as ae, RenderMessagesOptions as af, formatMessages as ag, renderMessages as ah, fixBrokenUth8String as ai, CurrencyInput as aj, currency as ak, Currency as al, DAY_MS as am, DayValue as an, DayInput as ao, dayYear as ap, dayMonth as aq, dayDay as ar, dayToParts as as, dayToDate as at, dayFromToday as au, dayFromAny as av, dayToDateGMT as aw, dayFromDate as ax, dayFromDateGMT as ay, dayToTimestamp as az, decrypt as b, UseDefer as b$, isRecord as b0, isString as b1, isNumber as b2, isInteger as b3, isSafeInteger as b4, isBoolean as b5, isNullOrUndefined as b6, jsonStringify as b7, getSecureRandom as b8, getSecureRandomIfPossible as b9, startSortWeight as bA, endSortWeight as bB, moveSortWeight as bC, sortedItems as bD, linkifyPlainText as bE, toHumanReadableUrl as bF, encodeQuery as bG, parseQuery as bH, ensureKey as bI, ensureKeyAsync as bJ, size as bK, last as bL, empty as bM, cloneObject as bN, cloneJsonObject as bO, memoize as bP, forTimes as bQ, regExpString as bR, regExpEscape as bS, XRX as bT, DisposerFunction as bU, Disposer as bV, Disposable as bW, useDispose as bX, UseDispose as bY, useDisposer as bZ, useDefer as b_, randomBoolean as ba, randomInt as bb, randomFloat as bc, between as bd, sum as be, avg as bf, isPrimeRX as bg, isPrime as bh, parseOrderby as bi, composeOrderby as bj, cmp as bk, sortedOrderby as bl, toValidFilename as bm, escapeRegExp as bn, RoundingMode as bo, isHalf as bp, isEven as bq, roundUp as br, roundDown as bs, roundHalfUp as bt, roundHalfOdd as bu, roundHalfAwayFromZero as bv, roundHalfDown as bw, roundHalfEven as bx, roundHalfTowardsZero as by, SortableItem as bz, csv as c, PubSub as c$, useTimeout as c0, useInterval as c1, useEventListener as c2, Mutex as c3, AsyncMutex as c4, useMutex as c5, useAsyncMutex as c6, PoolTaskFn as c7, PoolTaskState as c8, PoolTask as c9, LoggerMemoryHandler as cA, ChannelMessageEvent as cB, Channel as cC, LocalChannel as cD, fakeWorkerPair as cE, EmitterHandler as cF, EmitterAllHandler as cG, ListenerSignature as cH, DefaultListener as cI, Emitter as cJ, getGlobalEmitter as cK, messages as cL, lazyListener as cM, Encoder as cN, NoopEncoder as cO, JsonEncoder as cP, CryptoEncoder as cQ, MessageAction as cR, MessageResult as cS, Message as cT, MessagesOptions as cU, MessagesDefaultMethods as cV, MessagesMethods as cW, MessageDefinitions as cX, MessageHub as cY, createPromiseProxy as cZ, useMessageHub as c_, PoolTaskIdConflictResolution as ca, PoolTaskEvents as cb, usePool as cc, Pool as cd, createPromise as ce, sleep as cf, immediate as cg, timeoutReached as ch, timeout as ci, timoutError as cj, isTimeout as ck, tryTimeout as cl, waitOn as cm, isPromise as cn, promisify as co, TaskFn as cp, TaskEvents as cq, SerialQueue as cr, throttle as cs, debounce as ct, getGlobalContext as cu, isLocalHost as cv, LoggerConsoleHandler as cw, getNamespaceFilterString as cx, useNamespaceFilter as cy, useLevelFilter as cz, digest as d, usePubSub as d0, ArgumentsType as d1, ReturnType$1 as d2, RPCOptions as d3, RPCFn as d4, RPCReturn as d5, useRPC as d6, httpMethod as d7, fetchBasic as d8, fetchJson as d9, uuid32Decode as dA, uname as dB, qid as dC, uuidv4 as dD, suidBytes as dE, suid as dF, suidDate as dG, suidBytesDate as dH, fetchOptionsFormURLEncoded as da, fetchOptionsJson as db, fetchText as dc, getWindow as dd, getNavigator as de, getGlobal as df, detect as dg, isBrowser as dh, platform as di, useExitHandler as dj, MemStorageOptions as dk, MemStorage as dl, getTimestamp as dm, formatMilliseconds as dn, parseDate as dp, Json as dq, ObjectStorage as dr, NestedArray as ds, uuidBytes as dt, uuid32bit as du, uuid as dv, uuidEncode as dw, uuidDecode as dx, uuidB32 as dy, uuid32Encode as dz, encrypt as e, arrayUnique as f, arrayMinus as g, arrayUnion as h, arrayFlatten as i, arrayIntersection as j, arraySymmetricDifference as k, arrayRemoveElement as l, arraySetElement as m, arrayFilterInPlace as n, arrayToggleInPlace as o, arrayEmptyInPlace as p, arraySorted as q, randomUint8Array as r, arraySortedNumbers as s, arrayIsEqual as t, arrayShuffleInPlace as u, arrayShuffle as v, arrayShuffleForce as w, arrayRandomElement as x, arrayMax as y, arrayMin as z };
